#include <Ports.h>
#include <IRremote.h>
#include <RF12.h>
#include <RF12sio.h>

int led1Pin = A2;  //port 3 analogue
int led2Pin = A3; //port 4 analogue
int relay1Pin = 6; //port 3 DIO
int relay2Pin = 7; //POrt 4 DIO

const int inBufferLen = 255;
uint8_t inPtr = 0;
char inBuffer[inBufferLen+1];

#define ONE_WIRE_PIN  4 //port 1 DIO
#define ONE_WIRE_PIN2  5 //port 2 DIO

#define SERIAL  1   // set to 1 to also report readings on the serial port
#define DEBUG   0   // set to 1 to display each loop() run

#define MEASURE_PERIOD  600 // how often to measure, in tenths of seconds
#define RETRY_PERIOD    10  // how soon to retry if ACK didn't come in
#define RETRY_LIMIT     5   // maximum number of times to retry
#define ACK_TIME        10  // number of milliseconds to wait for an ack
#define REPORT_EVERY    1   // report every N measurement cycles

// The scheduler makes it easy to perform various tasks at various times:

enum { 
  MEASURE, REPORT, TASK_END };

static word schedbuf[TASK_END];
Scheduler scheduler (schedbuf, TASK_END);

// Other variables used in various places in the code:

static byte reportCount;    // count up until next report, i.e. packet send
static byte myNodeID;       // node ID used for this unit

// This defines the structure of the packets which get sent out by wireless:
struct {
int temp1:
  10; // temperature: -500..+500 (tenths)
int temp2:
  10; // temperature: -500..+500 (tenths)
byte relay1:
1;
byte relay2:
  1;
} 
payload;

// Conditional code, depending on which sensors are connected and how:

#if ONE_WIRE_PIN
#define TEMPERATURE_PRECISION 12

#include <OneWire.h>
#include <DallasTemperature.h>
// Setup a oneWire instance to communicate with any OneWire devices (not just Maxim/Dallas temperature ICs)
OneWire oneWire(ONE_WIRE_PIN);
// Pass our oneWire reference to Dallas Temperature. 
DallasTemperature sensors(&oneWire);
DeviceAddress deviceAddress;
#endif

#if ONE_WIRE_PIN2
OneWire oneWire2(ONE_WIRE_PIN2);
DallasTemperature sensors2(&oneWire2);
DeviceAddress deviceAddress2;
#endif

// UNUSED.
// wait a few milliseconds for proper ACK to me, return true if indeed received
//static byte waitForAck() {
//  MilliTimer ackTimer;
//  while (!ackTimer.poll(ACK_TIME)) {
//    if (rf12_recvDone() && rf12_crc == 0 &&
// see http://talk.jeelabs.net/topic/811#post-4712
//    rf12_hdr == (RF12_HDR_DST | RF12_HDR_CTL | myNodeID))
//      return 1;
//  }
//  return 0;
//}


void readSerial() {
  int inByte = Serial.read();
  if(inByte == '\r') { // CR to LF
    inByte = '\n';
  }
  switch(inByte) {
  case '\n':
    if(inPtr > 0) { 
      doCommand(); 
    }
    inPtr = 0;
    break;
  default:
    if(inPtr < inBufferLen) {
      inBuffer[inPtr] = inByte;
      inPtr++;
      inBuffer[inPtr] = '\0';
    }
  }
}

void doCommand() {
    if(strcmp("1 on", inBuffer) == 0) {
    setPort(1, 1);
  } 
   else if(strcmp("1 off", inBuffer) == 0) {
    setPort(1, 0);
  }
     else if(strcmp("2 on", inBuffer) == 0) {
    setPort(2, 1);
  }
     else if(strcmp("2 off", inBuffer) == 0) {
    setPort(2, 0);
  }
}
// periodic report, i.e. send out a packet and optionally report on serial port
static void doReport() {
  while (!rf12_canSend())
    rf12_recvDone();
  rf12_sendStart(0, &payload, sizeof payload);
#if SERIAL || DEBUG
  Serial.println(payload.relay1, DEC);
Serial.println(payload.relay1, DEC);
Serial.println(payload.temp, DEC);
#endif
}

static void doMeasure() {
  //    payload.lobat = rf12_lowbat();
#if ONE_WIRE_PIN
  sensors.requestTemperatures();
  payload.temp = sensors.getTempC(deviceAddress) * 10;
#endif

#if ONE_WIRE_PIN2
sensors2.requestTemperatures();
payload.temp2 = sensors2.getTempC(deviceAddress2) * 10;
#endif

}



static void setPort (byte relaytarget, byte targetstate) {
#if SERIAL || DEBUG
  Serial.println("Relay ");
  Serial.print(relaytarget, DEC);
  Serial.println(targetstate ? " set on" : " set off");
#endif

if (relaytarget == 1)
  {digitalWrite(relay1Pin, !targetstate); //logic is reversed for relay board
  digitalWrite(led1Pin, targetstate);   // sets the LED to match
	payload.relay1 = targetstate;}
else
  {digitalWrite(relay2Pin, !targetstate); //logic is reversed for relay board
  digitalWrite(led2Pin, targetstate);   // sets the LED to match
	payload.relay2 = targetstate;}
}

// Processing two bytes of RF12 data.
// XY
// X = Relay
// Y = STATE
static void doIncoming() {
  byte relay = rf12_data[0];
  if(relay < 1 || relay > 2) return;
  byte state = rf12_data[1];

  if(state > 1) state=1;       
  setPort(relay, state);
}


void setup () {

  pinMode(led1Pin, OUTPUT); 
  pinMode(relay1Pin, OUTPUT); 
  digitalWrite(relay1Pin, HIGH); //sets relay to off intially, reversed logic

  pinMode(led2Pin, OUTPUT); 
  pinMode(relay2Pin, OUTPUT); 
  digitalWrite(relay2Pin, HIGH); //sets relay to off intially, reversed logic

#if SERIAL || DEBUG
  Serial.begin(57600);
  Serial.print("\n[relay2Node.1]");
  myNodeID = rf12_config();
#else
  myNodeID = rf12_config(0); // don't report info on the serial port
#endif    

#if ONE_WIRE_PIN
  sensors.begin();
#if SERIAL || DEBUG
  Serial.print("Locating OneWire devices...");
  Serial.print("Found ");
  Serial.print(sensors.getDeviceCount(), DEC);
  Serial.println(" devices.");
#endif

  if (sensors.getAddress(deviceAddress, 0)) {
    sensors.setResolution(deviceAddress, TEMPERATURE_PRECISION);
  } 
  else {
#if SERIAL || DEBUG    
    Serial.println("Unable to find address for Device 0");
#endif
  }
#endif

#if ONE_WIRE_PIN2
sensors2.begin();
#if SERIAL || DEBUG
Serial.print("Locating OneWire2 devices...");
Serial.print("Found ");
Serial.print(sensors2.getDeviceCount(), DEC);
Serial.println(" devices.");
#endif
if (sensors2.getAddress(deviceAddress2, 0)) {
sensors2.setResolution(deviceAddress2, TEMPERATURE_PRECISION);
}
else {
#if SERIAL || DEBUG
Serial.println("Unable to find address for OneWire2 device");
#endif
}
#endif


  reportCount = REPORT_EVERY;     // report right away for easy debugging
  scheduler.timer(MEASURE, 0);    // start the measurement loop going
}

void loop () {
#if DEBUG
  Serial.print('.');
  delay(2);
#endif

  // process incoming data packet
  if (rf12_recvDone() && rf12_crc == 0 && rf12_len >0 && rf12_hdr == (RF12_HDR_DST | myNodeID)) {
#if SERIAL || DEBUG
    Serial.print("GOT");
    for (byte i = 0; i < rf12_len; ++i) {
      Serial.print("..");
      Serial.print((int)rf12_data[i]);
    }
    Serial.println();
#endif
    doIncoming();
  }

  while(Serial.available()) {
    readSerial();
  }

  switch (scheduler.poll()) {
  case MEASURE:
    // reschedule these measurements periodically
    scheduler.timer(MEASURE, MEASURE_PERIOD);

    doMeasure();

    // every so often, a report needs to be sent out
    if (++reportCount >= REPORT_EVERY) {
      reportCount = 0;
      scheduler.timer(REPORT, 0);
    }
    break;

  case REPORT:
    doReport();
    break;                
  }
}











